'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var pixelUtils = require('@mekari/pixel-utils');
var css = require('@emotion/css');
var pixelBox = require('@mekari/pixel-box');

const PseudoPropTypes = [Object, Array];
const MpControlBox = {
  name: 'MpControlBox',
  functional: true,
  inject: ['$pixelTheme'],
  props: {
    as: [String, Object],
    type: {
      type: String,
      default: 'checkbox'
    },
    size: {
      type: [Number, String, Array],
      default: 'auto'
    },
    _child: {
      type: PseudoPropTypes,
      default: () => ({
        opacity: 0
      })
    },
    _checkedAndChild: {
      type: PseudoPropTypes,
      default: () => ({
        opacity: 1
      })
    },
    _checkedAndDisabled: PseudoPropTypes,
    _checkedAndFocus: PseudoPropTypes,
    _checkedAndHover: PseudoPropTypes
  },

  render(h, {
    props,
    data,
    injections,
    slots
  }) {
    const {
      attrs
    } = data; // Inject theme

    const theme = injections.$pixelTheme(); // Parse child styles

    const checkedAndDisabled = `input[type=${props.type}]:checked:disabled + &, input[type=${props.type}][aria-checked=mixed]:disabled + &`;
    const checkedAndHover = `input[type=${props.type}]:checked:hover:not(:disabled) + &, input[type=${props.type}][aria-checked=mixed]:hover:not(:disabled) + &`;
    const checkedAndFocus = `input[type=${props.type}]:checked:focus + &, input[type=${props.type}][aria-checked=mixed]:focus + &`;
    const disabled = `input[type=${props.type}]:disabled + &`;
    const focus = `input[type=${props.type}]:focus + &`;
    const hover = `input[type=${props.type}]:hover:not(:disabled):not(:checked) + &`;
    const checked = `input[type=${props.type}]:checked + &, input[type=${props.type}][aria-checked=mixed] + &`;
    const invalid = `input[type=${props.type}][aria-invalid=true] + &`;
    const basePseudoAttrs = attrs && {
      [focus]: attrs._focus,
      [hover]: attrs._hover,
      [disabled]: attrs._disabled,
      [invalid]: attrs._invalid
    } || {};
    const controlBoxStyleObject = pixelUtils.composeSystem({ ...basePseudoAttrs,
      [checkedAndDisabled]: props._checkedAndDisabled,
      [checkedAndFocus]: props._checkedAndFocus,
      [checkedAndHover]: props._checkedAndHover,
      '& > *': props._child,
      [checked]: { ...(attrs && attrs._checked),
        '& > *': props._checkedAndChild
      }
    }, theme);
    const className = css.css(controlBoxStyleObject);
    return h(pixelBox.MpBox, { ...data,
      class: [className],
      props: {
        as: props.as
      },
      attrs: {
        display: 'inline-flex',
        alignItems: 'center',
        justifyContent: 'center',
        transition: 'all 120ms',
        flexShrink: '0',
        width: props.size,
        height: props.size,
        'aria-hidden': 'true',
        ...attrs,
        'data-pixel-component': 'MpControlBox'
      }
    }, slots().default);
  }

};

exports.MpControlBox = MpControlBox;
